//////////////////////////////////////////////////////////////////////////////////////////////////////
//																									//
//	Cliente TCP/IP en el sistema Bolt, por medio de un servidor TCP/IP remoto se controlan algunas	//
//	de las funciones del sitema Bolt, concretamente se controla el relevador y se puede leer la 	//
//	temperatura sel sensor DS18B20 incluido en el sistema Bolt.										//
//																									//
//	Si el sistema Bolt detecta que el servidor se ha desconectado se reinicia en aproximadamente 	//
//	2 minutos y comienza la secuencia de coneccion con el servidor, esto asegura que el sistema est//
//	conectado an el lugares remotos.																//					
//																									//
//	La conexin entre el cliente y el servidor TCP-IP tiene un timeout que fija el operador de 		//
//	telefona celular para el servicio GPRS (que en este proyecto es Telcel) y que es de alrededor 	//
//	de 1  2 minutos. Pasado ese tiempo, y en caso que no exista intercambio de mensajes entre el 	//
//	cliente y el servidor, la conexin se cierra en forma automtica, para evitar la desconexion	//
//	se enva desde el cliente TCP/IP el mensaje "..." cada 20 segundos esto mantiene abierta la 	//
//	conexin entre cliente y servidor de manera indefinida, tomando en cuenta que le costo del los	//
//	datos consumidos en muy reducido esto no incrementa de forma importante el costo que debe pa-	//
//	garse por el uso de la red celular.																//
//																									//
//	COMPILADOR: MPLAB v8.63 , C18 v3.40																//
//																									//
//  Moises Melendez, sesowm@hotmail.com																//
//																									//
//	Marzo 2015																						//
//																									//
//	Cdigo fuente que implementa un cliente TCP/IP en el sistema BOLT, la comunicacin se establece	//
//	mediante el modulo sim900, que es un dispositivos que permite la conexin a travs de GPRS en 	//
//	la red de telefona celular, en particular se utiliza el proveedor Telcel, se utiliza la 		//
//	siguiente informacin para poder acceder a la red del proveedor.								//
//																									//
//		APN: 				internet.virginmobile.mx 													//
//		Nombre de usuario: 	virgin 																//
//		Contrasea: 		virgin																//
//																									//
//	La aplicacion envia el siguinte mensaje para saber que se pueden enviar datos desde el servidor	// 																								//
//																									//
//************************************************************************************************* //
//																									//
//                    Cliente TCP/IP sim900-BOLT													//
//																									//
// Ejemplo de control del relevador y lectura de la temperatura en el sistema BOLT					//
//																									//
// Para encender el relevador: 		enciende relevador												//
// Para apagar el relevador: 		apaga relevador													//
// Para leer la temperatura: 		lee temperatura													//
// Para cerrar la conexion TCP/IP: 	cierra conexion													//
//																									//
//************************************************************************************************* //
//																									//
//	Escribe comando: 																				//
//																									//
//	Despues de este mesaje es posible enviar mensajes los que sern interpretados como comandos, 	//
//	si se reconoce el comando se ejecutar una accin en el sistema Bolt y este devolver un 		//
//	mensaje al servidor con la accin realizada. Se debe esperar aproximadamente tres segundos		//
//	antes de poder enviar un nuevo comando, debido al tiempo que requiere la red GSM/GPRS para		//
//	el envio de los datos a travs de la estructura celular.										//
//																									//
//	S: encender relevador																			//
//	No se reconoce comando																			//																		//
//	S: enciende relevador																			//
//	Relevador encendido!																			//
//	S: lee temperatura																				//
//	Temp:  21.3 C  																				//
//																									//
//	Conexion entre en modulo SIM900 y el sistema BOLT PIC18F2550									//
//																									//														
//		^												 	   										//
//	   | |												  											//
//	  _| |________				    	 ___________________			 ____________				//
//	 | | |		  |	    	     	 	|		   	 		| J1		|			 |				//
//	 | |_|		  |	Serial	     	J3	| 	   				|-----------| 9 Volts DC |				//		
//	 |  SIM900	  |	 port	   	  		|     BOLT SYSTEM 	|  			|____________|				//
//	 |		   TXD|---------------------|RX      			|										//
//	 |		   RXD|---------------------|TX        			|										//
//	 |_________GND|---------------------|GND________________|										//
//		|									  ||||||												//
//		|							DB9		 _____________											//
//		|									|	LCD		  |											//
//		|									|_____________|											//
//	   _|____________																				//			
//	  |				 |																				//
//	  |	3.9 Volts DC |																				//
//	  |______________|																				//
//																									//
//////////////////////////////////////////////////////////////////////////////////////////////////////
//																									//
#define SERVER_IP  	"\"puntoflotante.ddns.net\""  // IP de la PC que corre la aplicacion de servidor//
#define SERVER_PORT			   "\"2020\""	      // Numero de puerto utilizado por el servidor		//	
//																									//
//**************************************************************************************************//

#include <p18cxxx.h>
#include <delays.h>
#include <usart.h>
#include <string.h>
#include <stdio.h>		//libreria sprintf()
#include <stdlib.h>		//librerias atoi(),atof()
#include "18F2550BOLT.h"
#include "PUERTO-SERIAL-BOLT.h"
#include "ONEWIRE-BOLT.h"

#define CONV const far rom char *
#define INCOMING_SIZE 220		// Tamao del arreglo de datos recibidos
#define OK			0
#define ERROR		1
#define NO_ANSWER	2
#define TIEMPO		20			// Tiempo es segundos para enviar el mensaje "..."
#define SLEEP		while(1){}

// Prototipo de funciones
void rx_handler (void);											// Prototipo de ISR USART
void initConfiguration(void);									// Configuraciones iniciales
void writeLCDmsg(const far rom char * message);					// Escribe una cadena en el LCD
void commandProcess(void);										// Funcion que procesa los comandos
unsigned char sendATcommand(const far rom char * command, const far rom char * answer, unsigned int delay, unsigned int timeout);
																// Envia los comandos AT al modulo sim900
void sendSTATUS2LCD(unsigned char STATUS, const far rom char * message2LCD);	// Escribe el estado del proceso de conexion
void sendATdata(const far rom char *data);						// Envia datos al modulo sim900
unsigned char conect2SERVER(void);								// Funcion que conecta el modulo sim900 al servidor
void noConection(void);											// No es posible establecer la conexion
void sendSTATUScommand(const far rom char * command);			// Funcio de peticion del estado de la conexin

// Variables globales
volatile static char serialIncomingData[INCOMING_SIZE];			// Datos provenientes del puerto serie
volatile static unsigned char INDEX=0, INTERRUPT=0;				// Variables para el control de la interrupcion

extern void _startup( void ); // See c018i.c in your C18 compiler dir 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 		// Vector de reset
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code rx_interrupt = 0x000808					// Vector de interrupcion de alta prioridad
void rx_int (void)
{
	_asm goto rx_handler _endasm
}

#pragma code

//
// Funcion para el manejo de la interrupcion de alta prioridad
//
#pragma interrupt rx_handler	
void rx_handler (void)
{
	serialIncomingData[INDEX++] = ReadUSART();	// Lee el buffer de entrada del puerto seria del pic
	PIR1bits.RCIF = 0;							// Limpia la bandera de interrupcion
	INTERRUPT=1;								// La interrupcion ha ocurrido
}
 
void main()						// Funcion principal
{
	unsigned char STATUS;		// Variable de control para monitorear el estado de la conexion
	initConfiguration();		// Configuracines iniciales 
	STATUS=conect2SERVER();		// Conecta el sitema BOLTd-sim900 con un servidor remoto
	if(STATUS==OK)				// Si la conexion se ha establecido
		commandProcess();		// Procesa los comandos que envia el servidor
	else						// Si la conexion no se ha establecido 
		noConection();			// Envia un mensaje de error
} 

//
// Si la conexion no se ha establecido envia
// un mensaje de error a traves del LCD del sistema BOLT
//
void noConection(void)						
{											
	writeLCDmsg((CONV)"  No se puede   ");	
	delay_ms(800);
	writeLCDmsg((CONV)"conectar con el ");
	delay_ms(800);
	writeLCDmsg((CONV)"servidor TCP/IP ");
	delay_ms(800);
	writeLCDmsg((CONV)"  Error TCP/IP  ");
	SLEEP;									// Loop infinito.
}

//
//	Funcion que envia todos los comandos AT necesarion para establecer la comunicacion con un 
//	servidor remoto a traves de GPRS en el modulo sim900, se requiere de conocer la IP y el 
//	numero de puerto del servidor al que se requiere conectar. Ademas monitorea constantemente
//	el estado de la conexion, si no es posible establecerla debido a que alguno de los comandos
//	no puede procesarse la funcion retorna un valor de ERROR.
//
unsigned char conect2SERVER(void)
{
	unsigned char STATUS;
	writeLCDmsg((CONV)"  BOLT-SIM900   ");
	delay_ms(1000);
	writeLCDmsg((CONV)"   Aplicacion   ");
	delay_ms(1000);
	writeLCDmsg((CONV)"     TCP/IP     ");
	delay_ms(1000);
	STATUS=sendATcommand((CONV)"AT+CIPSHUT",(CONV)"SHUT OK",3,2);
	delay_ms(2000);
	STATUS=sendATcommand((CONV)"AT",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT command   ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CNUM=?",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CNUM      ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CREG?",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CREG      ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CSQ",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"    AT+CSQ      ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CMEE=2",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CMEE=2    ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CMGF=1",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CMGF=1    ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CSCS=\"GSM\"",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"  AT+CSCS=\"GSM\"  ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CGATT=1",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CGATT=1   ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CGATT?",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CGATT     ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CSQ",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"     AT+CSQ     ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+COPS?",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"    AT+COPS     ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CIPMUX=0",(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CIPMUX    ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CIPMODE=0",(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CIPMODE   ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CIPCSGP=1,\"internet.virginmobile.mx\"",(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"  AT+CIPCSGP    ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CLPORT=\"TCP\","SERVER_PORT,(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CLPORT    ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CSTT=\"internet.virginmobile.mx\",\"virgin\",\"virgin\"",(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"    AT+CSTT     ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CIPSRIP=1",(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CIPSRIP   ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CIICR",(CONV)"OK",8,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CIICR     ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CIFSR",(CONV)"AT+CIFSR",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CIFSR     ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CIPSTART=\"TCP\","SERVER_IP","SERVER_PORT,(CONV)"CONNECT OK",15,2);
	sendSTATUS2LCD(STATUS,(CONV)"  AT+CIPSTART   ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CIPSTATUS",(CONV)"CONNECT OK",3,2);
	sendSTATUS2LCD(STATUS,(CONV)"  AT+CIPSTATUS  ");
	if(STATUS!=OK) return ERROR;
	STATUS=sendATcommand((CONV)"AT+CIPSEND",(CONV)">",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"  Data send     ");
	if(STATUS!=OK) return ERROR;
	sendATdata((CONV)"******************************************************************************************************");
	sendATdata((CONV)"");
	sendATdata((CONV)"                    Cliente TCP/IP sim900-BOLT");
	sendATdata((CONV)"");
	sendATdata((CONV)" Ejemplo de control del relevador y lectura de la temperatura en el sistema BOLT");
	sendATdata((CONV)"");
	sendATdata((CONV)" Para encender el relevador:      enciende relevador");
	sendATdata((CONV)" Para apagar el relevador:        apaga relevador");
	sendATdata((CONV)" Para leer la temperatura:        lee temperatura");
	sendATdata((CONV)" Para cerrar la conexion TCP/IP:  cierra conexion");
	sendATdata((CONV)"");
	sendATdata((CONV)"******************************************************************************************************");
	while(BusyUSART());
	putcUSART((char)26);
	delay_ms(8000);
	STATUS=sendATcommand((CONV)"AT+CIPSEND",(CONV)">",2,2);
	if(STATUS!=OK) return ERROR;
	sendATdata((CONV)"Escribe comando: ");
	while(BusyUSART());
	putcUSART((char)26);
	delay_ms(2000);
	return OK;		
}

//
// Configuraciones iniciales para el sistema Bolt, cofigura el LCD, los puertos, el relevador
// y el puerto serie en hardware
//
void initConfiguration(void)	
{								
	int initIndex;			// Variable local para limpiar el arreglo que contendra los datos recibidos
	init_bolt();			
	InitLCD();
	activa_serial();		
	RCONbits.IPEN = 1;		// Habilita prioridad en las interrupciones
	IPR1bits.RCIP = 1;		// Alta prioridad para la interrupcion por llegada de datos
	INTCONbits.GIEH = 1;	// Habilita todas las interrupciones de alta prioridad
	for(initIndex=0;initIndex<INCOMING_SIZE;initIndex++) serialIncomingData[initIndex]=0;
	WDTCONbits.SWDTEN = 1; 	// Habilita watchdog 
}

//
// Envia los datos al modulo sim900 para ser enviados al servidor 
//
void sendATdata(const far rom char *data)	
{
	int i;
	for(i=0; i<strlenpgm(data); i++) 
	{
		while(BusyUSART());				// Espera a que el puerto serie este desocupado
		putcUSART(data[i]);				// Escribe cada uno de los caracteres al puerto serie
	}
	while(BusyUSART());
	putcUSART('\r');					// Retorno de carro
	while(BusyUSART());
	putcUSART((char)10);
	delay_ms(200);
}

//
// Escribe la cadena "message" al LCD
//
void writeLCDmsg(const far rom char * message)		
{
	unsigned char i;
	ClearScreen();							// Limpia la pantalla
	GotoXY(0,0);							// Ve a la posicion (0,0)
	for(i=0; i<8; i++) PutLCD(message[i]);	// Imprime cada uno de los caracteres de la cadena del argumento
	GotoXY(1,0);
	for(i=8; i<strlenpgm(message);i++) PutLCD(message[i]);
}

//
//	Funcion que procesa los mensajes provenientes del servidor, si es un comando reconocido devuelve un mensaje 
//	indicando el efecto del comando.
//
void commandProcess(void)		
{
	int k, nMSG=0;
	int initIndex;
	float temperatura;
	int entero, decimal;
	unsigned char STATUS;
	char on[]=		"enciende relevador";	// Cadenas de caracteres para el reconocimiento de comandodos
	char off[]=		"apaga relevador";		
	char cierra[]=	"cierra conexion";
	char tempe[]=	"lee temperatura";
	char close[]=	"CLOSED";
	char ok[]=		"CONNECT OK";
	char temp[20];
	INDEX=0; 
	INTERRUPT=0;
	while(1)
	{
		ClrWdt();	
		if(INTERRUPT)						// Si la interrupcion ha ocurrido se procesan los datos.
		{
			if(strstr(serialIncomingData,ok)) 
			{
				//writeLCDmsg((CONV)" Send ... token ");
				STATUS=sendATcommand((CONV)"AT+CIPSEND",(CONV)">",2,2);
				if(STATUS==OK)
				{
					sendATdata((CONV)"...");
					while(BusyUSART());
					putcUSART((char)26);
					delay_ms(2500);
				}
			}
			else if(strstr(serialIncomingData,on)) 
			{
				RELE=1;
				writeLCDmsg((CONV)"  Relevador ON  ");
				STATUS=sendATcommand((CONV)"AT+CIPSEND",(CONV)">",2,2);
				if(STATUS==OK)
				{
					sendATdata((CONV)"Relevador encendido!");
					while(BusyUSART());
					putcUSART((char)26);
					delay_ms(3000);
				}
			}
			else if(strstr(serialIncomingData,off)) 
			{
				RELE=0;
				writeLCDmsg((CONV)" Relevador OFF  ");
				STATUS=sendATcommand((CONV)"AT+CIPSEND",(CONV)">",2,2);
				if(STATUS==OK)
				{
					sendATdata((CONV)"Relevador apagado!");
					while(BusyUSART());
					putcUSART((char)26);
					delay_ms(3000);
				}
			}
			else if(strstr(serialIncomingData,tempe))
			{
				temperatura=ds1820_read();
				entero=temperatura;						// Parte entera
				decimal=(temperatura-entero)*10;		// Parte decimal
				sprintf(temp,(CONV)"Temp: %3d.%d C  \n\r",entero,decimal);		// Convierte a una cadena de caracteres
				ClearScreen();
				GotoXY(0,0);
				for(k=0; k<8; k++) PutLCD(temp[k]);
				GotoXY(1,0);
				for(k=8; k<11;k++) PutLCD(temp[k]);
				PutLCD(32);							//espacio
				PutLCD(214);						//simbolo de grados
				PutLCD('C');
				PutLCD(32);							//espacio
				PutLCD(32);							//espacio
				STATUS=sendATcommand((CONV)"AT+CIPSEND",(CONV)">",2,2);
				if(STATUS==OK)
				{
					for(k=0; k<strlen(temp); k++) 
					{
						while(BusyUSART());
						putcUSART(temp[k]);
					}
					while(BusyUSART());
					putcUSART('\r');
					delay_ms(200);
					while(BusyUSART());
					putcUSART((char)26);
					delay_ms(3000);
				}
			}
			else if(strstr(serialIncomingData,cierra))
			{
				STATUS=sendATcommand((CONV)"AT+CIPSHUT",(CONV)"SHUT OK",3,2);	// Cierra la conexion TCP/IP
				if(STATUS==OK)	writeLCDmsg((CONV)"Conexion cerrada");
				WDTCONbits.SWDTEN = 0; 											// Deshabilita watchdog 
				SLEEP;
			}
			else if(strstr(serialIncomingData,close))
			{
				writeLCDmsg((CONV)" El servidor se ");
				delay_ms(2000);
				writeLCDmsg((CONV)"ha desconectado!");
				SLEEP;
			}	
			else 
			{
				STATUS=sendATcommand((CONV)"AT+CIPSEND",(CONV)">",2,2);				// Si el comando no se reconoce
				if(STATUS==OK)														// se envia el mensaje 
				{																	// "No se reconoce comando"
					sendATdata((CONV)"No se reconoce comando");
					while(BusyUSART());
					putcUSART((char)26);
					delay_ms(2000);
				}
				writeLCDmsg((CONV)"Ninguna accion  ");
			}
			INDEX=0;
			INTERRUPT=0;
			for(k=0;k<INCOMING_SIZE;k++) serialIncomingData[k]=0;
		}
		delay_ms(1000);
		nMSG++;
		if(nMSG>=TIEMPO)			// Tiempo en segundos para reenviar el mensaje "..."
		{
			sendSTATUScommand((CONV)"AT+CIPSTATUS");
			delay_ms(1000);
			nMSG=0;
		} 
	}
}

//
// Funcion de peticion del estado de la conexin al modulo sim900
//
void sendSTATUScommand(const far rom char * command)
{
	int i;
	for(i=0; i<strlenpgm(command); i++) 
	{
		while(BusyUSART());		// Espera a que este disponible el modulo USART
		putcUSART(command[i]);	// envia cada uno de los caracteres del comando AT
	}
	putcUSART('\r');	
	delay_ms(200);
}

//	
//	Funcion que envia un comando AT al modulo sim900, recibe el comando AT, la respuesta esperada, el retardo para el
//	procesamiento del comando AT por parte del modulo sim900, y un tiempo limite, si el modulo sim900 no responde en 
//	el lapso de tiempo indicado por este numero la funcion retorna el valor NO_ANSWER
//
unsigned char sendATcommand(const far rom char * command, const far rom char * answer, unsigned int delay, unsigned int timeout)
{
	unsigned long int time;
	int i;
	for(i=0; i<strlenpgm(command); i++) 
	{
		while(BusyUSART());		// Espera a que este disponible el modulo USART
		putcUSART(command[i]);	// envia cada uno de los caracteres del comando AT
	}
	putcUSART('\r');			// equivale a pulsar ENTER
	delay_ms(delay*1000);
	for(time=0;time<(timeout*200000);time++)
	{
		if(INTERRUPT)
		{
			if(strstrrampgm(serialIncomingData,answer)) 
			{	
				INDEX=0;
				INTERRUPT=0;
				for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
				return OK;
			}
			else 
			{	
				INDEX=0;
				INTERRUPT=0;
				for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
				return ERROR;
			}
		}
		else continue;
	}
	INDEX=0;
	INTERRUPT=0;
	for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
	return NO_ANSWER;
}

//
//	Funcion que envia un mensaje en el LCD indicando si un comando AT ha sido procesado correctamente
//
void sendSTATUS2LCD(unsigned char STATUS, const far rom char * message2LCD)
{
	writeLCDmsg(message2LCD);
	delay_ms(500);
	switch(STATUS){
	case OK:{ 
		writeLCDmsg((CONV)"       OK       ");
		delay_ms(500);
		break;}
	case ERROR:{
		writeLCDmsg((CONV)"      Error     ");
		delay_ms(500);
		break;}
	case NO_ANSWER:{
		writeLCDmsg((CONV)" Sin respuesta  ");
		delay_ms(500);
		break;}
	default: 
		break;}
}